//**************************************************************************
//			                                                                  	
//    KNCT-MMEdu for Arduino Nano Every       
//    (c) Kiyoteru Hayama(Kumamoto National College of Technology)
//                                            
//  This sketch is using "megaAVR_TimerInterrupt" library
//                                                    	
//  Select menu: set-sw.
//  LED: indicates mode 00 to 11  (binary)        
//  2024.05.07  Ver 1.0
//**************************************************************************
#define USE_TIMER_1     true     //  write before inclunde
#define TIMER1_INTERVAL_MS   1   // 
#include "megaAVR_TimerInterrupt.h"

#define GOAL_Y 7  // goal position
#define GOAL_X 7  // goal position

#define PT_FR  A0                         // front right sensor, analog input
#define PT_FL  A1                         // front left sensor, analog input
#define PT_R   A2                         // right sensor, analog input
#define PT_L   A3                         // left sensor, analog input
#define GYRO   A4                         // for Gyro, analog input
#define LEDOUT 5                        // LED output signal for wall detection(AN5 used for digital input 19)
#define L0  10                         // disp LED L0 
#define L1  9                         // disp LED L1
#define SET    4                         // set-switch, digital input
#define START  3                         // start-switch, digital input

// sensitivity
#define KFR   1                      // front right sensor value in normal micromouse position 
#define KFL   0.9                      // front left sensor value in normal micromouse position 
#define KR    1.5                       // right sensor value in normal micromouse position 
#define KL    0.2 

// run parameters
#define LSPD    4                       // timer count for low speed 
#define HSPD    3                       // timer count for high speed 
#define STEP1   500                      // number of step for 1 maze area
#define SLOWSTP 20                       // for slowstart 
#define R90     200                      // number of step for 90 degree right turn 
#define L90     200                      //  number of step for 90 degree left turn 
#define R180    400                      //  number of step for 180 degree u-turn  
#define DISFR   320                      // front right sensor value in normal micromouse position 
#define DISFL   256                      // front left sensor value in normal micromouse position 
#define DISR    200                       // right sensor value in normal micromouse position 
#define DISL    200                      // left sensor value in normal micromouse position    
#define DISFMAX 50                       // threshold of sensor value for front wall detection
#define DISRMAX 50                       // threshold of sensor value for right  wall detection
#define DISLMAX 50                       // threshold of sensor value for left wall detection

// pattern table for stepping motor 
//const byte RMOTOR[]={0x01, 0x02, 0x08, 0x04, 0x00};  // energize pattern for motor 
//const byte LMOTOR[]={0x08, 0x04, 0x02, 0x01, 0x00};  // energize pattern for motor 
const byte RMOTOR[]={0x03, 0x0a, 0x0c, 0x05, 0x00};  // energize pattern for motor 
const byte LMOTOR[]={0x0c, 0x06, 0x03, 0x09, 0x00};  // energize pattern for motor 


const byte DtoR[]={0,2,4,0,8,0,0,0,1};   // table indicating to the right direction
const byte DtoL[]={0,8,1,0,2,0,0,0,4};   // table indicating to the left direction      

byte pmode=0;                            //  mode

// It is necessary to define as a Volatile  when the variable used in interrupt.
volatile byte modeR=0, modeL=0;                   // run forward both motor   
volatile int stepR, stepL;                        // varilable for set step of motor
volatile byte patR=0, patL=0;                     // index of motor pattern  
volatile int cntR, cntL;                          // count of motor steps 
volatile int sensFR, sensFL, sensR, sensL;        // sensor values
volatile int sensFRB, sensFLB, sensRB, sensLB;    // sensor values during turn-off the LED 
volatile byte timR=0, timL=0;                     // waiting timer for motors
volatile byte timS;                               // waiting timer for sensors 
volatile byte fS=0;                               // flag for control of distanse from R,L walls
volatile byte fR=0, fL=0;                         // flag of R, L motors, 0: low speed, 1:hight speed

long dtime;

union {                                  // struct and union define for access map 
       byte all;                         // map access by 1 byte 
       struct { byte n:1;                // 1 bit for north wall (0:no wall, 1:exist wall)  
                byte e:1;                // 1 bit for east wall  (0:no wall, 1:exist wall)   
                byte s:1;                // 1 bit for south wall (0:no wall, 1:exist wall)  
                byte w:1;                // 1 bit for west wall  (0:no wall, 1:exist wall)    
                byte d:4;                // 4bit for history  
              };
        } mmap[16][16]; 

//--------------------------------------------------------------------------
//	initialze 	                                                  
//--------------------------------------------------------------------------
void setup()
{
  int i,j;

  PORTA.DIR |= B00001111;  // portAの下位4bitをまとめて扱う。A5,A4,D7,D2 が　PA3,2,1,0 for motor R
  PORTE.DIR |= B00001111;  // portEの下位4bitをまとめて扱う。D8,D13,D12,D11 が　PE3,2,1,0, for motor L
 
  pinMode(SET, INPUT_PULLUP);
  pinMode(START, INPUT_PULLUP);
  pinMode(L0, OUTPUT); // LED L0
  pinMode(L1, OUTPUT); // LED L1
  pinMode(LEDOUT, OUTPUT);             // LED output for wall detection
  digitalWrite(LEDOUT, LOW);           // LED turn off

  // initialize map
  for (i=0; i<16;i++) for (j=0;j<16;j++) mmap[i][j].all=0;   // clear map
  for (i=0; i<16;i++){
     mmap[i][0].w=1; mmap[i][15].e=1;                         // set east and west wall 
     mmap[0][i].s=1; mmap[15][i].n=1; }                       // set north and south wall 
  mmap[0][0].e=1;    
 
  ITimer1.init();
	ITimer1.attachInterruptInterval(TIMER1_INTERVAL_MS , SensAndMotor);

  // open serial port for PC connection (read value of sensors)
  Serial.begin(9600);                  
}

//--------------------------------------------------------------------------
//	main loop 	                                                  
//--------------------------------------------------------------------------
void loop()
{
  while (digitalRead(START)==HIGH){
    if (digitalRead(SET)==LOW) {      // wait set switch 
      delay(10);                      // wait 10ms to avoid bounds.
      while(digitalRead(SET)==LOW);
      delay(10);                      // wait 10ms to avoid bounds.
      pmode++; if (pmode>3) pmode=0;
    }
    dispLED(pmode);
  }
  dispLED(0);
  delay(500);                           // wait 0.5 sec 

  // go selected functions
  switch(pmode){
    /*
    case  0: run_step(); break;          // run 1 area step 
    case  1: run_R90();  break;          // 90 deg. turn right 
    case  2: run_L90();  break;          // 90 deg. turn left     
    case  3: run_R180(); break;          // u-turn       
     */ 
    case  0: check_sens();   break;       // check sensors  
    case  1: run_Turn(0);    break;       // go forward and u-turn
    case  2: run_Hidarite(); break;       // left hand method   
    case  3: run_saitan();   break;       // fast run for minimum route   

  }
}





